package com.shellever.plugin;

import static com.shellever.plugin.Utils.*;

class Main {

    private final Send send;
    private final long groupID;
    private final String msg;
    private final String nick;
    private final long qq;
    private final long atqq;
    private final String at;
    private final String root;
    private final String ssPath;
    private final String atSsPath;
    private final String itemPath;
    private final String atItemPath;
    private final String nickPath;
    private final String atNickPath;
    private final String kjRankPath;
    private final String timePath;
    private final String timesPath;
    private final String dailyAttendancePath;
    private final String messagePath;
    private final String bossPath;
    private final String nowSeasonPath;
    private final int nowSeason;
    private final int mySeason;
    private final int atSeason;
    private final long time;
    private final String timeStr;

    Main(Send send, long groupID, String msg, String nick, long qq, long atqq) {
        this.send = send;
        this.groupID = groupID;
        this.msg = msg;
        this.nick = nick;
        this.qq = qq;
        this.atqq = atqq;
        // this.at = "$At " + qq + "$@" + nick + " \\r";
        this.at = "◇" + nick + "◇\\r";
        this.time = System.currentTimeMillis();
        this.timeStr = timeToStr(time);
        this.root = ROOT + "鲲/";
        this.ssPath = root + "属性/" + qq;// 赛季、等级、鲲名
        this.atSsPath = root + "属性/" + atqq;
        this.itemPath = root + "道具/" + qq;// 萌泪币、改名卡、……
        this.atItemPath = root + "道具/" + atqq;
        this.nickPath = root + "昵称/" + groupID + "/" + qq;
        this.atNickPath = root + "昵称/" + groupID + "/" + atqq;
        this.kjRankPath = root + "排行/" + groupID;
        this.timePath = root + "时间/时间/" + qq;
        this.timesPath = root + "时间/次数/" + qq;
        this.dailyAttendancePath = root + "签到/" + qq;
        this.messagePath = root + "消息/";
        this.bossPath = root + "Boss";
        this.nowSeasonPath = root + "当前赛季";
        this.nowSeason = getInt(nowSeasonPath, 1, 1);
        this.mySeason = getInt(ssPath, "赛季", 1);
        this.atSeason = getInt(atSsPath, "赛季", 1);
    }

    /**
     * 正则处理消息.
     * .表示除\n以外的任何字符
     * *表示0个到无穷个
     * +表示1个到无穷个
     * |表示左侧或右侧任意一个（可以与括号配合使用）
     * [0-9]表示任意一个数字
     * [0-9A-Za-z]表示任意一个数字或字母
     */
    void processMsg() {
        if (nowSeason != mySeason) {
            seasonSettlement();
            return;
        }
        set(nickPath, 1, nick);
        if (msg.matches("帮助|养鲲手册|菜单")) {
            send("养鲲：从无尽之海抓一条奇怪的鲲\\r"
                    + "属性：查看鲲的属性\\r"
                    + "查看：查看别人鲲的属性，栗子【查看@xx】\\r"
                    + "洗练：洗练指定属性，栗子【洗练攻防血 12】\\r"
                    + "进击：互相挑战，栗子【进击@xx】\\r"
                    + "挑战：全群挑战Boss，可得丰厚奖励\\r"
                    + "Boss：查看Boss属性\\r"
                    + "排行：查看本群鲲界等级前十\\r"
                    + "背包：查看自己的所有道具\\r"
                    + "命名：消耗改名卡将鲲界的鲲改名，栗子【命名戮鲲】\\r"
                    + "赠送：赠送萌泪币，栗子【赠送200@xxx】\\r"
                    + "商城：查看道具列表，显示买/卖价格\\r"
                    + "购买/出售：买卖道具，栗子【购买洗练卡 3】\\r"
                    + "签到：每日签到，可得一定量萌泪币，连续签到奖励更多\\r"
                    + "当前为S" + nowSeason + "赛季！"
            );
        } else if (msg.matches("消息条数")) {
            msgInfo();
        } else if (msg.matches("重置")) {
            resetRobotMsg();
        } else if (msg.matches("开启新赛季")) {
            setNewSeason();
        } else if (msg.matches("更改.+ +[0-9]+")) {
            int a = Integer.parseInt(msg.substring(msg.lastIndexOf(' ') + 1));
            if (a != 0) {
                change(msg.substring(2, msg.indexOf(' ')), a);
            }
        } else if (msg.matches("养鲲|摸鲲|抓鲲|捕鲲")) {
            mkInit();
        } else if (msg.matches("属性")) {
            attribute();
        } else if (msg.matches("查看@.+")) {
            if (atSeason != nowSeason) {
                send(at + "对方还未开启新赛季呢！");
                return;
            }
            viewOthersAttribute();
        } else if (msg.matches("洗练.+ +[0-9]+")) {
            int a = Integer.parseInt(msg.substring(msg.lastIndexOf(' ') + 1));
            if (a != 0) {
                washout(msg.substring(2, msg.indexOf(' ')), a);
            }
        } else if (msg.matches("进击@.+")) {
            if (atSeason != nowSeason) {
                send(at + "对方还未开启新赛季呢！");
                return;
            }
            attackOthersInit();
        } else if (msg.matches("挑战")) {
            attackBossInit();
        } else if (msg.matches("(查看|)[Bb]oss(属性|)")) {
            viewBossAttribute();
        } else if (msg.matches("排行|排行榜")) {
            rankingList();
        } else if (msg.matches("道具|背包")) {
            bag();
        } else if (msg.matches("命名.+")) {
            name(msg.substring(2));
        } else if (msg.matches("赠送[0-9]+@.+")) {
            if (atSeason != nowSeason) {
                send(at + "对方还未开启新赛季呢！");
                return;
            }
            giveMoney(Integer.parseInt(msg.substring(2, msg.indexOf('@'))));
        } else if (msg.matches("商城")) {
            shop();
        } else if (msg.matches("购买.+ +[0-9]+")) {
            int a = Integer.parseInt(msg.substring(msg.lastIndexOf(' ') + 1));
            if (a != 0) {
                buy(msg.substring(2, msg.indexOf(' ')), a);
            }
        } else if (msg.matches("出售.+ +[0-9]+")) {
            int a = Integer.parseInt(msg.substring(msg.lastIndexOf(' ') + 1));
            if (a != 0) {
                sell(msg.substring(2, msg.indexOf(' ')), a);
            }
        } else if (msg.matches("购买.+")) {
            buy(msg.substring(2), 1);
        } else if (msg.matches("出售.+")) {
            sell(msg.substring(2), 1);
        } else if (msg.matches("签到")) {
            dailyAttendance();
        }
    }

    private void send(String str) {
        String day = timeStr.substring(0, 10);
        String hour = timeStr.substring(11, 13);
        int dayNum = getInt(messagePath + day, "all") + 1;
        int hourNum = getInt(messagePath + day, hour) + 1;
        int robotMsgNum = getInt(messagePath + "/机器人", 1) + 1;
        set(messagePath + day, "all", dayNum);
        set(messagePath + day, hour, hourNum);
        set(messagePath + "机器人", 1, robotMsgNum);
        send.s(str + "\\r" + robotMsgNum + " / " + dayNum
                + (robotMsgNum > 1000 ?
                "\\r抗议！！！我要下班！！！\\r（快找萌泪换机器人！）" : ""));
    }

    /**
     * 赛季结算.
     */
    private void seasonSettlement() {
        if (nowSeason == mySeason) {
            send(at + "已开启S" + nowSeason + "赛季！\\r无需进行赛季结算~");
            return;
        }
        int level = getInt(ssPath, "等级");
        if (level == 0) {
            send(at + "S" + nowSeason + "赛季现已开启！\\r快去养鲲吧！");
            set(ssPath, "赛季", nowSeason);
            return;
        }
        int newLevel = 0;
        int addMoney = 0;
        for (int i = mySeason; i < nowSeason; i++) {
            newLevel = getNewLv(level);
            addMoney += level / 3;
        }
        set(ssPath, "赛季", nowSeason);
        set(ssPath, "等级", newLevel);
        set(itemPath, "萌泪币", getInt(itemPath, "萌泪币") + addMoney);
        getOnTheList(qq, newLevel);
        String name = getString(ssPath, "鲲名", "木有名字");
        send(at + "S" + nowSeason + "赛季现已开启！\\r" +
                name + "鲲初始等级为" + newLevel + "\\r" +
                "获得萌泪币" + addMoney + "枚\\r快去养鲲吧！");
        set(itemPath, "洗练卡", getInt(itemPath, "洗练卡") + 3);
        washout("攻防血", 1);
    }

    /**
     * 赛季结算等级继承.
     *
     * @param level 原先鲲的等级
     * @return 继承后的等级
     */
    private static int getNewLv(int level) {
        int newLevel = 0;
        int i = 10;
        for (; i > 1; i--) {
            int q = (10 - i) * (10 - i) * 300 + 3000;
            if (level <= q) {
                break;
            } else {
                newLevel = newLevel + q * i / 10;
                level = level - q;
            }
        }
        return newLevel + level * i / 10;
    }

    private void msgInfo() {
        String day = timeStr.substring(0, 10);
        int dayNum = getInt(messagePath + day, "all") + 1;
        StringBuilder s = new StringBuilder("今日消息总数：" + dayNum + "\\r");
        for (int i = 0; i < 24; i++) {
            String hour = i <= 9 ? "0" + i : "" + i;
            s.append(hour).append("h").append(getInt(messagePath + day, hour));
            if (i % 4 == 3 && i != 23) {
                s.append("\\r");
            } else {
                s.append(" ");
            }
        }
        send(s.toString());
    }

    private void resetRobotMsg() {
        delete(messagePath + "机器人");
    }

    private void setNewSeason() {
        if (qq != 605738729L) {
            return;
        }
        int newSeason = nowSeason + 1;
        set(nowSeasonPath, "1", newSeason);
        delete(root + "排行");
        delete(bossPath);
        send(at + "已开启S" + newSeason + "赛季！");
    }

    private void change(String key, int newValue) {
        if (qq != 605738729L) {
            return;
        }
        switch (key) {
            case "萌泪币":
                set(itemPath, "萌泪币", newValue);
                break;
            case "等级":
                set(ssPath, "等级", newValue);
                break;
            default:
        }
    }

    /**
     * 判断摸鲲条件，并进行时间处理.
     */
    private void mkInit() {
        long oldTime = getLong(timePath, "摸鲲");
        String oldDate = timeToStr(oldTime).substring(0, 10);
        String date = timeStr.substring(0, 10);
        int times = getInt(timesPath, "摸鲲次数") + 1;
        if (oldDate.equals(date)) {
            // 相同天要判断能不能摸
            long targetTime = oldTime + getFibonacci(times) * 60000;
            String targetDate = timeToStr(targetTime).substring(0, 10);
            if (!targetDate.equals(date)) {
                send(at + "你已经很累了，明日再来摸鲲吧！");
                return;
            }
            int timeDiff = (int) (targetTime - time) / 1000;
            if (timeDiff > 0) {
                send(at + "下次摸鲲时间为" + timeToStr(targetTime).substring(11) +
                        "，还需等待" + timeDiffToStr(timeDiff) + "~");
                return;
            }
        }
        // 不同天，所以必定是新的一天；或者是相同天但是可以继续
        if (qq != 605738729L && qq != 81038735L) {
            set(timePath, "摸鲲", time);
            set(timesPath, "摸鲲次数", times);
        }
        mk();
    }

    /**
     * 摸鲲.
     */
    private void mk() {
        int level = getInt(ssPath, "等级");
        int money = getInt(itemPath, "萌泪币");
        if (level == 0) {
            String newName = getName();
            set(ssPath, "鲲名", newName);
            set(ssPath, "等级", 1);
            set(ssPath, "攻击", 1);
            set(ssPath, "防御", 1);
            set(ssPath, "血量", 1);
            send(at + "你孤身一人去无尽之海，发现一只小鲲崽遗弃在海岸边。\\r"
                    + "你看它非常可爱，便给它取名为" + newName + "鲲，决定抚养它成为最强的鲲王。\\r"
                    + "只是这无尽之海非常广阔，你还需要抓更多的鲲喂给它，使其慢慢成长....");
            return;
        }
        String name = getString(ssPath, "鲲名");
        int event = getRandomInt(0, 99);
        if (event < 10) {
            // 无事发生（0-9）
            send(at + "你去无尽之海，结果什么都没摸到！\\r"
                    + "只好带着" + name + "鲲回去了。");
        } else if (event < 20) {
            // 被偷走钱（10-19）
            int subMoney = getNormalDistributionInt(money * 5 / 1000, money * 15 / 1000);
            set(itemPath, "萌泪币", money - subMoney);
            send(at + "你去无尽之海，结果什么都没摸到！\\r"
                    + "一摸背包，发现萌泪币少了" + subMoney + "！\\r"
                    + "你很生气，却不知是谁偷了你的钱，只好带着" + name + "鲲回去了。");
        } else if (event < 30) {
            // 等级降低（20-29）
            int subLevel = getNormalDistributionInt(40, 100 + level / 500);
            set(ssPath, "等级", level - subLevel);
            send(at + "你去无尽之海，忽然一只巨鲲向你咬来！\\r"
                    + "这时，" + name + "鲲挺身而出，替你挡下了这一击，"
                    + "自己却受伤，等级降低了" + subLevel + "！\\r"
                    + "你立刻离开无尽之海，带着" + name + "鲲回去休养了。");
        } else {
            // 正常摸鲲（30-99）
            int addLevel = event < 36 ?
                    getNormalDistributionInt(200, 500 + level / 60)
                    : getNormalDistributionInt(40, 100 + level / 300);
            int addMoney = event > 33 && event < 40 ?
                    getNormalDistributionInt(addLevel * 5, addLevel * 15)
                    : getNormalDistributionInt(addLevel, addLevel * 3);
            set(ssPath, "等级", level + addLevel);
            set(itemPath, "萌泪币", money + addMoney);
            getOnTheList(qq, level + addLevel);
            if (event < 34) {
                // 鲲等级提高（30-33）
                send(at + "你去无尽之海，抓了很多很多的鲲！\\r"
                        + "你将大部分鲲喂给" + name + "鲲，等级提高" + addLevel + "！\\r"
                        + "剩余的鲲都被你卖掉，获得" + addMoney + "枚萌泪币！");
            } else if (event < 36) {
                // 鲲等级、萌泪币提高（34-35）
                send(at + "你去无尽之海，不仅抓了很多鲲，还捞上来一个大宝箱！\\r"
                        + "你将鲲喂给" + name + "鲲，等级提高" + addLevel + "！\\r"
                        + "你又打开宝箱，发现里面竟然有" + addMoney + "枚萌泪币！");
            } else if (event < 40) {
                // 萌泪币提高（36-39）
                send(at + "你去无尽之海，抓了一些鲲，还捞上来一个小宝箱！\\r"
                        + "你将鲲喂给" + name + "鲲，等级提高" + addLevel + "！\\r"
                        + "你又打开宝箱，发现里面竟然有" + addMoney + "枚萌泪币！");
            } else {
                // 正常（40-99）
                send(at + "你去无尽之海，抓了一些鲲！\\r"
                        + "你将大部分鲲喂给" + name + "鲲，等级提高" + addLevel + "！\\r"
                        + "剩余的鲲都被你卖掉，获得" + addMoney + "枚萌泪币！");
            }
        }
    }

    private String getName() {
        StringBuilder name = new StringBuilder();
        int charNum = getRandomInt(1, 4);
        for (int i = 0; i < charNum; i++) {
            name.append(getRandomChar());
        }
        return name.toString();
    }

    /**
     * 上榜前，有两种情况：在/不在排行榜中。
     * 初始排名定为11，向上移动，直至前一位等级不低于自己等级。
     *
     * @param qq    可能上榜的qq
     * @param level 该qq的等级
     */
    private void getOnTheList(long qq, int level) {
        long phbQQ = getLong(kjRankPath, 10);
        if (phbQQ != qq &&
                getInt(root + "属性/" + phbQQ, "等级") >= level) {
            return;
        }
        int initRank = 10;// 初始排名
        for (int i = 1; i < 10; i++) {
            phbQQ = getLong(kjRankPath, i);
            if (phbQQ == qq) {
                initRank = i;
                break;
            }
        }
        for (int i = initRank; i > 1; i--) {
            phbQQ = getLong(kjRankPath, i - 1);
            int previousLv = getInt(root + "属性/" + phbQQ, "等级");// 上一位的等级
            if (level > previousLv) {
                set(kjRankPath, i, phbQQ);
            } else {
                set(kjRankPath, i, qq);
                return;
            }
        }
        set(kjRankPath, 1, qq);
    }

    /**
     * 属性
     */
    private void attribute() {
        int level = getInt(ssPath, "等级");
        if (level == 0) {
            send(at + "在无尽之海，也许你会有所发现....\\r指令提示：摸鲲");
            return;
        }
        String name = getString(ssPath, "鲲名");
        int atk = getInt(ssPath, "攻击");
        int def = getInt(ssPath, "防御");
        int hp = getInt(ssPath, "血量");
        String atkGrade = grade(atk, level, 1, 1.5);
        String defGrade = grade(def, level, 0.6, 0.9);
        String hpGrade = grade(hp, level, 4, 6);
        send(at + name + "鲲正在到处游弋。\\r"
                + "等级：" + level + "\\r"
                + "血量：" + hp + "（" + hpGrade + "）\\r"
                + "攻击：" + atk + "（" + atkGrade + "）\\r"
                + "防御：" + def + "（" + defGrade + "）");
    }

    private String grade(int attributeValue, int level, double min, double max) {
        if (attributeValue == (int) (level * max)) {
            return "MAX";
        }
        double cha = level * (max - min);
        double overPart = attributeValue - level * min;

        String str;
        if (overPart < cha / 8) {
            str = "E";
        } else if (overPart < cha * 2 / 8) {
            str = "D";
            overPart -= cha / 8;
        } else if (overPart < cha * 3 / 8) {
            str = "C";
            overPart -= cha * 2 / 8;
        } else if (overPart < cha * 4 / 8) {
            str = "B";
            overPart -= cha * 3 / 8;
        } else if (overPart < cha * 5 / 8) {
            str = "A";
            overPart -= cha * 4 / 8;
        } else if (overPart < cha * 6 / 8) {
            str = "S";
            overPart -= cha * 5 / 8;
        } else if (overPart < cha * 7 / 8) {
            str = "SS";
            overPart -= cha * 6 / 8;
        } else {
            str = "SSS";
            overPart -= cha * 7 / 8;
        }
        cha /= 8;
        if (overPart < cha / 3) {
            return str + "-";
        } else if (overPart < cha * 2 / 3) {
            return str;
        } else {
            return str + "+";
        }
    }

    private void viewOthersAttribute() {
        int level = getInt(atSsPath, "等级");
        if (level == 0) {
            send(at + "对方还没鲲呢！");
            return;
        }
        int ckk = getInt(itemPath, "查看卡");
        if (ckk <= 0) {
            send(at + "你没有查看卡，无权查看对方信息！");
            return;
        }
        set(itemPath, "查看卡", ckk - 1);
        String name = getString(atSsPath, "鲲名");
        int atk = getInt(atSsPath, "攻击");
        int def = getInt(atSsPath, "防御");
        int hp = getInt(atSsPath, "血量");
        String atkGrade = grade(atk, level, 1, 1.5);
        String defGrade = grade(def, level, 0.6, 0.9);
        String hpGrade = grade(hp, level, 4, 6);
        String atNick = getString(atNickPath, 1);
        send(at + name + "鲲\\r"
                + "主人：" + atNick + "\\r"
                + "等级：" + level + "\\r"
                + "血量：" + hp + "（" + hpGrade + "）\\r"
                + "攻击：" + atk + "（" + atkGrade + "）\\r"
                + "防御：" + def + "（" + defGrade + "）");
    }

    /**
     * 洗练.
     *
     * @param type  要洗练的属性
     * @param times 每个属性洗练的次数
     */
    private void washout(String type, int times) {
        int level = getInt(ssPath, "等级");
        if (level == 0) {
            send(at + "在无尽之海，也许你会有所发现....\\r指令提示：摸鲲");
            return;
        }
        boolean atk = type.contains("攻");
        boolean def = type.contains("防");
        boolean hp = type.contains("血");
        int xlkNum = getInt(itemPath, "洗练卡");
        int useNum = 0;
        useNum = atk ? useNum + times : useNum;
        useNum = def ? useNum + times : useNum;
        useNum = hp ? useNum + times : useNum;
        if (xlkNum < useNum) {
            send(at + "洗练卡不足" + useNum + "！");
            return;
        }
        set(itemPath, "洗练卡", xlkNum - useNum);
        String s = getString(ssPath, "鲲名") + "鲲正在洗练....";
        if (atk) {
            int maxNum = 0;
            for (int i = 0; i < times; i++) {
                maxNum = Math.max(maxNum, (int) getNormalDistributionDouble(level * 1.0, level * 1.5));
            }
            s = s + "\\r洗练攻击" + times + "次，最高" + maxNum;
            set(ssPath, "攻击", maxNum);
        }
        if (def) {
            int maxNum = 0;
            for (int i = 0; i < times; i++) {
                maxNum = Math.max(maxNum, (int) getNormalDistributionDouble(level * 0.6, level * 0.9));
            }
            s = s + "\\r洗练防御" + times + "次，最高" + maxNum;
            set(ssPath, "防御", maxNum);
        }
        if (hp) {
            int maxNum = 0;
            for (int i = 0; i < times; i++) {
                maxNum = Math.max(maxNum, (int) getNormalDistributionDouble(level * 4.0, level * 6.0));
            }
            s = s + "\\r洗练血量" + times + "次，最高" + maxNum;
            set(ssPath, "血量", maxNum);
        }
        if (atk || def || hp) {
            send(at + s);
        } else {
            send(at + "你要洗练神马？\\r大声点，人家听不清啦！\\r" +
                    "栗子【洗练攻防血 3】\\r（至少洗练一项）");
        }
    }

    private void attackOthersInit() {
        int myLevel = getInt(ssPath, "等级");
        int atLevel = getInt(atSsPath, "等级");
        if (myLevel == 0) {
            send(at + "在无尽之海，也许你会有所发现....\\r指令提示：摸鲲");
            return;
        } else if (atLevel == 0) {
            send(at + "别人还没鲲呢，是想欺负萌新吗？");
            return;
        } else if (atLevel * 1.5 < myLevel) {
            send(at + "你的鲲等级太高了，是想欺负萌新吗？");
            return;
        } else if (atqq == qq) {
            send(at + getString(ssPath, "鲲名")
                    + "鲲表示不想打自己，并向你丢了一个白眼！");
            return;
        }

        long oldTime = getLong(timePath, "进击");
        String oldDate = timeToStr(oldTime).substring(0, 10);
        String date = timeStr.substring(0, 10);
        int times = getInt(timesPath, "进击次数") + 1;
        if (oldDate.equals(date)) {
            // 相同天要判断能不能摸
            long targetTime = oldTime + getFibonacci(times) * 60000;
            String targetDate = timeToStr(targetTime).substring(0, 10);
            if (!targetDate.equals(date)) {
                send(at + "你已经很累了，明日再来进击吧！");
                return;
            }
            int timeDiff = (int) (targetTime - time) / 1000;
            if (timeDiff > 0) {
                send(at + "下次进击时间为" + timeToStr(targetTime).substring(11) +
                        "，还需等待" + timeDiffToStr(timeDiff) + "~");
                return;
            }
        }
        // 不同天，所以必定是新的一天；或者是相同天但是可以继续
        if (qq != 605738729L && qq != 81038735L) {
            set(timePath, "进击", time);
            set(timesPath, "进击次数", times);
        }
        attackOthers();
    }

    private void attackOthers() {
        int num1 = getRandomInt(100, 110);
        int num2 = getRandomInt(90, 100);
        int myAtk = getInt(ssPath, "攻击") * num1 / 100;
        int atAtk = getInt(atSsPath, "攻击") * num2 / 100;
        int myDef = getInt(ssPath, "防御") * num1 / 100;
        int atDef = getInt(atSsPath, "防御") * num2 / 100;
        int myHp = getInt(ssPath, "血量") * num1 / 100;
        int atHp = getInt(atSsPath, "血量") * num2 / 100;
        int myAtkTimes = 0;
        int atAtkTimes = 0;
        int myAllDam = 0;
        int atAllDam = 0;
        String myName = getString(ssPath, "鲲名");
        String atName = getString(atSsPath, "鲲名");
        StringBuilder str = new StringBuilder(at + myName + "鲲 VS " + atName + "鲲\\r");
        boolean isWin;

        while (true) {
            int myDam = Math.max(myAtk - atDef, 1);
            myAtkTimes++;
            myAllDam += myDam;
            if (atHp <= myAllDam) {
                str.append(myName).append("鲲攻击").append(myAtkTimes).append("次\\r")
                        .append("共造成伤害").append(myAllDam).append("点\\r");
                if (myAtkTimes > 1) {
                    str.append(atName).append("鲲反击").append(atAtkTimes).append("次\\r")
                            .append("共造成伤害").append(atAllDam).append("点\\r");
                }
                str.append("获胜啦！\\r");
                isWin = true;
                break;
            }
            int atDam = Math.max(atAtk - myDef, 1);
            atAtkTimes++;
            atAllDam += atDam;
            if (myHp <= atAllDam) {
                str.append(myName).append("鲲攻击").append(myAtkTimes).append("次\\r")
                        .append("共造成伤害").append(myAllDam).append("点\\r")
                        .append(atName).append("鲲反击").append(atAtkTimes).append("次\\r")
                        .append("共造成伤害").append(atAllDam).append("点\\r")
                        .append("失败了！\\r");
                isWin = false;
                break;
            }
        }
        int myLevel = getInt(ssPath, "等级");
        int atLevel = getInt(atSsPath, "等级");
        int myAdd = isWin ? getAdd(myLevel) : (int) (getAdd(myLevel) * 0.7);
        int atAdd = (int) (getAdd(atLevel) * 0.25);
        myLevel += myAdd;
        atLevel += atAdd;
        send(str + "经过磨炼，\\r" +
                myName + "鲲等级增加" + myAdd + "！\\r" +
                atName + "鲲等级增加" + atAdd + "！");
        set(ssPath, "等级", myLevel);
        set(atSsPath, "等级", atLevel);
        getOnTheList(qq, myLevel);
        getOnTheList(atqq, atLevel);
    }

    private int getAdd(int level) {
        long phbQQ = getLong(kjRankPath, 1);
        int phbMaxLevel = getInt(root + "属性/" + phbQQ, "等级");
        double fj1 = phbMaxLevel * 0.4;
        double fj2 = phbMaxLevel * 0.65;
        double fj3 = phbMaxLevel * 0.9;
        if (level < fj1) {
            return getNormalDistributionInt(400, 800);
        } else if (level < fj2) {
            return getNormalDistributionInt(200, 400);
        } else if (level < fj3) {
            return getNormalDistributionInt(100, 200);
        } else {
            return getNormalDistributionInt(50, 100);
        }
    }

    private void attackBossInit() {
        int myLevel = getInt(ssPath, "等级");
        if (myLevel == 0) {
            send(at + "在无尽之海，也许你会有所发现....\\r指令提示：摸鲲");
            return;
        }
        int ticket = getInt(itemPath, "挑战券");
        if (ticket <= 0) {
            send(at + "你还没挑战券呢！");
            return;
        }
        long oldTime = getLong(timePath, "挑战");
        String oldDate = timeToStr(oldTime).substring(0, 10);
        String date = timeStr.substring(0, 10);
        int times = getInt(timesPath, "挑战次数") + 1;
        if (oldDate.equals(date)) {
            // 相同天要判断能不能摸
            long targetTime;
            if (times == 2) {
                targetTime = oldTime + 3600000 * 3;
            } else if (times == 3) {
                targetTime = oldTime + 3600000 * 6;
            } else {
                targetTime = oldTime + 3600000 * 24;
            }
            String targetDate = timeToStr(targetTime).substring(0, 10);
            if (!targetDate.equals(date)) {
                send(at + "你已经很累了，明日再来挑战吧！");
                return;
            }
            int timeDiff = (int) (targetTime - time) / 1000;
            if (timeDiff > 0) {
                send(at + "下次挑战时间为" + timeToStr(targetTime).substring(11) +
                        "，还需等待" + timeDiffToStr(timeDiff) + "~");
                return;
            }
        }
        // 不同天，所以必定是新的一天；或者是相同天但是可以继续
        if (qq != 605738729L && qq != 81038735L) {
            set(timePath, "挑战", time);
            set(timesPath, "挑战次数", times);
        }
        set(itemPath, "挑战券", ticket - 1);
        attackBoss();
    }

    private void attackBoss() {
        int bossHp = getInt(bossPath, "血量");
        if (bossHp <= 0) {
            newBoss();
            bossHp = getInt(bossPath, "血量");
        }
        int bossAtk = getInt(bossPath, "攻击");
        int bossDef = getInt(bossPath, "防御");
        int myAtk = getInt(ssPath, "攻击");
        int myDef = getInt(ssPath, "防御");
        int myHp = getInt(ssPath, "血量");
        int myAtkTimes = 0;
        int bossAtkTimes = 0;
        int myAllDam = 0;
        int bossAllDam = 0;
        String myName = getString(ssPath, "鲲名");
        String bossName = getString(bossPath, "鲲名");
        double attributeIncrease;
        if (getRandomDouble(1, 20) < 7) {
            attributeIncrease = getNormalDistributionDouble(1, 13);
            if (attributeIncrease > 7) {
                attributeIncrease = 14 - attributeIncrease;
            }
        } else {
            attributeIncrease = getNormalDistributionDouble(-6, 20);
            if (attributeIncrease < 7) {
                attributeIncrease = 14 - attributeIncrease;
            }
        }
        myAtk = (int) (myAtk * attributeIncrease);
        myDef = (int) (myDef * attributeIncrease * 0.1);
        myHp = (int) (myHp * attributeIncrease * 0.5);
        StringBuilder str = new StringBuilder(at + myName + "鲲 VS " + bossName + "鲲\\r" +
                "随机加成倍数：" + String.format("%.2f", attributeIncrease) + "\\r");
        boolean isWin;
        while (true) {
            int myDam = Math.max(myAtk - bossDef, 1);
            myAtkTimes++;
            myAllDam += myDam;
            if (bossHp <= myAllDam) {
                str.append(myName).append("鲲攻击").append(myAtkTimes).append("次\\r")
                        .append("共造成伤害").append(myAllDam).append("点\\r");
                if (myAtkTimes > 1) {
                    str.append(bossName).append("鲲反击").append(bossAtkTimes).append("次\\r")
                            .append("共造成伤害").append(bossAllDam).append("点\\r");
                }
                str.append("获胜啦！\\r");
                isWin = true;
                break;
            }
            int atDam = Math.max(bossAtk - myDef, 1);
            bossAtkTimes++;
            bossAllDam += atDam;
            if (myHp <= bossAllDam) {
                str.append(myName).append("鲲攻击").append(myAtkTimes).append("次\\r")
                        .append("共造成伤害").append(myAllDam).append("点\\r")
                        .append(bossName).append("鲲反击").append(bossAtkTimes).append("次\\r")
                        .append("共造成伤害").append(bossAllDam).append("点\\r")
                        .append("失败了！\\r");
                isWin = false;
                break;
            }
        }
        set(bossPath, "血量", bossHp - myAllDam);
        int bossLevel = getInt(bossPath, "等级");
        int getMoney = isWin ? (int) ((long) myAllDam * 10 / bossLevel + 66666)
                : (int) ((long) myAllDam * 10 / bossLevel + 1000);
        set(itemPath, "萌泪币", getInt(itemPath, "萌泪币") + getMoney);
        send(str.append("获得了").append(getMoney).append("枚萌泪币！").toString());
    }

    private void newBoss() {
        String[] allGroup = getAllFileName(root + "排行");
        int bossLevel = 1000;
        for (String group : allGroup) {
            long phbQQ = getLong(root + "排行/" + group, 1);
            int thisPhbMaxLevel = getInt(root + "属性/" + phbQQ, "等级");
            if (thisPhbMaxLevel > bossLevel) {
                bossLevel = thisPhbMaxLevel;
            }
        }
        set(bossPath, "鲲名", getName());
        set(bossPath, "等级", bossLevel);
        int bossHp = bossLevel * 10000;
        int bossAtk = bossLevel * 2;
        int bossDef = bossLevel;
        set(bossPath, "攻击", bossAtk);
        set(bossPath, "防御", bossDef);
        set(bossPath, "血量", bossHp);
    }

    private void viewBossAttribute() {
        int ckk = getInt(itemPath, "查看卡");
        if (ckk <= 0) {
            send(at + "你没有查看卡，无权查看对方信息！");
            return;
        }
        int hp = getInt(bossPath, "血量");
        if (hp <= 0) {
            newBoss();
            hp = getInt(bossPath, "血量");
        }
        set(itemPath, "查看卡", ckk - 1);
        String name = getString(bossPath, "鲲名");
        int atk = getInt(bossPath, "攻击");
        int def = getInt(bossPath, "防御");
        send(at + "Boss " + name + "鲲\\r" +
                "攻击：" + atk + "\\r" +
                "防御：" + def + "\\r" +
                "剩余血量：" + hp);
    }

    private void rankingList() {
        StringBuilder str1 = new StringBuilder();// 1-5名
        StringBuilder str2 = new StringBuilder();// 6-10名
        for (int i = 1; i <= 10; i++) {
            long rankqq = getLong(kjRankPath, i);
            if (rankqq == DEFAULT_LONG) {
                break;
            }
            int ranklevel = getInt(root + "属性/" + rankqq, "等级");
            String rankname = getString(root + "属性/" + rankqq, "鲲名");
            String ranknick = getString(root + "昵称/" + groupID + "/" + rankqq, 1);
            if (i <= 5) {
                str1.append(i == 1 ? "" : "\\r")
                        .append(i).append(" Lv.").append(ranklevel)
                        .append(" ").append(rankname).append("鲲\\r")
                        .append(ranknick).append("（").append(rankqq).append("）");
            } else {
                str2.append(i == 6 ? "" : "\\r")
                        .append(i).append(" Lv.").append(ranklevel)
                        .append(" ").append(rankname).append("鲲\\r")
                        .append(ranknick).append("（").append(rankqq).append("）");
            }
        }
        if (!str1.toString().equals("")) {
            send(str1.toString());
        } else {
            send(at + "当前无人上榜！");
        }
        if (!str2.toString().equals("")) {
            send(str2.toString());
        }
    }

    private void bag() {
        send(at + "改名卡：" + getInt(itemPath, "改名卡") + "张\\r" +
                "洗练卡：" + getInt(itemPath, "洗练卡") + "张\\r" +
                "挑战券：" + getInt(itemPath, "挑战券") + "张\\r" +
                "查看卡：" + getInt(itemPath, "查看卡") + "张\\r" +
                "萌泪币：" + getInt(itemPath, "萌泪币") + "枚");
    }

    private void name(String newName) {
        int level = getInt(ssPath, "等级");
        if (level == DEFAULT_INT) {
            send(at + "在无尽之海，也许你会有所发现....\\r指令提示：摸鲲");
            return;
        }
        newName = newName.replaceAll("\\s*", "");
        if (newName.substring(newName.length() - 1).equals("鲲")) {
            newName = newName.substring(0, newName.length() - 1);
        }
        if (newName.equals("")) {
            send(at + "指令提示：命名xx（鲲）");
            return;
        }
        int gmk = getInt(itemPath, "改名卡");
        if (gmk <= 0) {
            send(at + getString(ssPath, "鲲名") + "鲲摇了摇头，表示不喜欢这个名字！");
        }
        set(ssPath, "鲲名", newName);
        set(itemPath, "改名卡", gmk - 1);
        send(at + getString(ssPath, "鲲名") + "鲲高兴地绕着你转了两圈，" +
                "看来它很喜欢这个名字！\\r" +
                "以后它就叫" + newName + "鲲啦！");
    }

    private void giveMoney(int giveMoney) {
        int myMoney = getInt(itemPath, "萌泪币");
        if (giveMoney > myMoney) {
            send(at + "钱不够！\\r你在想peach？");
            return;
        }
        int atMoney = getInt(atItemPath, "萌泪币");
        set(itemPath, "萌泪币", myMoney - giveMoney);
        set(atItemPath, "萌泪币", atMoney + giveMoney);
        send(at + "成功赠送了" + giveMoney + "枚萌泪币！");
    }

    private void shop() {
        send("———— 商城 ————\\r" +
                "改名卡：18888 / 15110\\r" +
                "洗练卡：   30 /    24\\r" +
                "挑战券：  100 /     -\\r" +
                "查看卡：  100 /    80\\r" +
                "左边购买价格，右边出售价格");
    }

    private void buy(String itemName, int buyNum) {
        int money = getInt(itemPath, "萌泪币");
        int price;
        switch (itemName) {
            case "改名卡":
                price = 18888;
                break;
            case "洗练卡":
                price = 30;
                break;
            case "挑战券":
            case "查看卡":
                price = 100;
                break;
            default:
                send(at + "你要买神马东西？\\r大声一点，人家听不清啦！");
                return;
        }
        if (money < price * buyNum) {
            send(at + "现有萌泪币：" + money + "\\r不足" + price * buyNum + "，无法购买！");
            return;
        }
        int itemNum = getInt(itemPath, itemName);
        set(itemPath, itemName, itemNum + buyNum);
        set(itemPath, "萌泪币", money - price * buyNum);
        send(at + "成功购买" + itemName + "×" + buyNum + "！\\r" +
                "花费" + price * buyNum + "枚萌泪币！\\r" +
                "现有萌泪币：" + (money - price * buyNum) + "枚");
    }

    private void sell(String itemName, int soldNum) {
        int price;
        switch (itemName) {
            case "改名卡":
                price = 15110;
                break;
            case "洗练卡":
                price = 24;
                break;
            case "查看卡":
                price = 80;
                break;
            default:
                send(at + "你要卖神马东西？\\r大声一点，人家听不清啦！");
                return;
        }
        int itemNum = getInt(itemPath, itemName);
        if (itemNum < soldNum) {
            send(at + itemName + "在哪？\\r你在想peach？");
            return;
        }
        int money = getInt(itemPath, "萌泪币");
        set(itemPath, itemName, itemNum - soldNum);
        set(itemPath, "萌泪币", money + price * soldNum);
        send(at + "成功出售" + itemName + "×" + soldNum + "！\\r" +
                "获得" + price * soldNum + "枚萌泪币！\\r" +
                "现有萌泪币：" + (money + price * soldNum) + "枚");
    }

    private void dailyAttendance() {
        String today = timeStr.substring(0, 10);
        boolean signedIn = getInt(dailyAttendancePath, today) == 1;
        int times = getInt(dailyAttendancePath, "签到次数");
        if (signedIn) {
            send(at + "共签到" + times + "次\\r" +
                    "连续签到" + getInt(dailyAttendancePath, "连续签到天数") + "天\\r" +
                    "明天再来签到吧！");
            return;
        }
        set(dailyAttendancePath, today, 1);
        times++;
        set(dailyAttendancePath, "签到次数", times);
        String yesterday = timeToStr(time - 86400000L).substring(0, 10);
        int continuous = getInt(dailyAttendancePath, yesterday) == 1 ?
                getInt(dailyAttendancePath, "连续签到天数") + 1 : 1;
        set(dailyAttendancePath, "连续签到天数", continuous);
        int money = getInt(itemPath, "萌泪币");
        int getMoney;
        if (continuous % 7 == 0) {
            getMoney = 6666;
        } else if (continuous % 7 <= 3) {
            getMoney = 666;
        } else if (continuous % 7 <= 5) {
            getMoney = 1666;
        } else {
            getMoney = 2666;
        }
        set(itemPath, "萌泪币", money + getMoney);
        send(at + "共签到" + times + "次\\r" +
                "连续签到" + continuous + "天\\r" +
                "获得" + getMoney + "枚萌泪币！");
    }

}
